#! /usr/bin/python

"""Module for creating Wrestler objects.

This module builds the Wrestler class with validation for its fields. When
validation is stronger than simple type validation, the Mark class is used
in replace of traditional str or int classes to track accuracy.

Example:
    wrestler = Wrestler(name='Anthony, Nick', team="Eagles", grade=Mark('Sr.'))

"""

from typing import Dict, Optional, Union

import attr
from attr.validators import instance_of

from wrestling import base


def convert_to_title(name: str) -> str:
    """Makes a string title-ized.

    Args:
        name: Any string.

    Returns:
        str: Capitalized and white-spaced stripped string.

    """
    return name.title().strip()


@attr.s(kw_only=True, auto_attribs=True, order=True, eq=True, frozen=True, slots=True)
class Wrestler(object):
    """Wrestler object.

    Args:
        name (str): Name of the wrestler. Ex: Last, First.
        team (str): Team the wrestler represents.
        grade (Union[Mark, None]): Grade/eligibility of the wrestler, default to None.

    """

    name: str = attr.ib(
        converter=convert_to_title, validator=instance_of(str), order=True
    )
    team: str = attr.ib(
        converter=convert_to_title, validator=instance_of(str), order=False
    )
    _grade: Optional[Union[base.Mark, None]] = attr.ib(
        default=None, order=False, eq=False,
    )

    def __attrs_post_init__(self):
        """Post init function to call Mark input handlers."""
        self.grade_input_handler()

    @property
    def grade(self) -> str:
        """Eligibility of athlete.

        Returns:
            str: Grade/Eligbility of athlete.
        """
        if self._grade:
            return str(self._grade.tag)
        return str(self._grade)

    def grade_input_handler(self) -> None:
        """Function to manage validity of 'grade' input attribute via Mark class."""
        if self._grade:
            if self._grade.tag not in base.YEARS:
                message = (
                    f"Invalid year, expected one of {*base.YEARS,}, "
                    f"got {self._grade.tag}."
                )
                self._grade.isvalid = False
                self._grade.msg = message

    def to_dict(self) -> Dict[str, str]:
        """Creates a dictionary representation of an Wrestler instance.

        Returns:
            Dict: Dictionary with the name, team, and grade of the Wrestler instance.

        """
        return dict(name=self.name, team=self.team, grade=self.grade)



from wrestling.base import Result, Mark, CollegeLabel, HSLabel
import enum

def test_result():
    results_set = {
        "Loss Fall", "Loss Tech", "Loss Major", "Loss Dec",
        "Win Fall", "Win Tech", "Win Major", "Win Dec",
        "No Contest",
    }
    for result in Result:
        assert isinstance(result.value, int)
        assert result.text in results_set
        assert isinstance(result.win, bool)
        assert isinstance(result.bonus, bool)
        assert isinstance(result.pin, bool)
        assert isinstance(result.team_points, int)
        assert result.team_points in {0, 3, 4, 5, 6}


def test_mark():
    mark1 = Mark(12)
    mark2 = Mark('15')
    mark3 = Mark('hello')
    
    for mark in [mark1, mark2, mark3]:
        assert isinstance(mark.tag, (str, int))


def test_college_label():
    label = CollegeLabel('R01')
    assert isinstance(label.tag, (str, int))
    assert label.point_value in range(5)


def test_college_label():
    label = CollegeLabel('N3')
    assert isinstance(label.tag, (str, int))
    assert label.point_value in range(5)


#! /usr/bin/python

"""Module for base classes and globals used throughout the project.

This module contains the Years dictionary, Result enumeration class,
the Mark class which is foundational to all other classes in the project,
and the CollegeLabel and HSLabel classes inheriting from the Mark class.

"""

import enum
from typing import Dict, Set, Union

import attr
from attr.validators import instance_of

# eligibility
YEARS = {
    "Fr.",
    "So.",
    "Jr.",
    "Sr.",
    "RS Fr.",
    "RS So.",
    "RS Jr.",
    "RS Sr.",
    "7",
    "8",
    "9",
    "10",
    "11",
    "12",
}
"""str: Module level variable containing string variants
of acceptable years of eligibility.

"""


# result section
class Result(enum.IntEnum):
    """Enumeration class for match Results.

    This class contains information on Results
    structured as an enumeration. There are additional properties
    for other metrics that can be identified based on the Result.

    Args:
        enum (IntEnum): IntEnum super class.

    Returns:
        name (str): String abbreviated representation of the result.
        value (int): Numeric representation of the result.

    """

    WD = 1
    WM = 2
    WT = 3
    WF = 4
    LD = -1
    LM = -2
    LT = -3
    LF = -4
    # anytime the match didn't fully end (disq, default, forfeit, inj, etc)
    NC = 0

    @property
    def text(self) -> str:
        """Expanded text representation of result.

        Returns:
            str: Full text of result.

        """
        text = ""
        if self.name == "NC":
            text = "No Contest"
        elif self.name == "WD":
            text = "Win Dec"
        elif self.name == "WM":
            text = "Win Major"
        elif self.name == "WT":
            text = "Win Tech"
        elif self.name == "WF":
            text = "Win Fall"
        elif self.name == "LD":
            text = "Loss Dec"
        elif self.name == "LM":
            text = "Loss Major"
        elif self.name == "LT":
            text = "Loss Tech"
        elif self.name == "LF":
            text = "Loss Fall"
        return text

    @property
    def win(self) -> bool:
        """Win or Loss.

        Returns:
            bool: True if Win, False if Loss.

        """
        return True if self.value > 0 else False

    @property
    def bonus(self) -> bool:
        """Identifies whether Result is considered bonus or not.

        Returns:
            bool: True if considered bonus, else False.

        """
        return True if self.value > 1 or self.value < -1 else False

    @property
    def pin(self) -> bool:
        """Identifies if Result is a pin variation.

        Returns:
            bool: True if method of result is Fall, else False.

        """
        return True if self.value == 4 else False

    @property
    def team_points(self) -> int:
        """Calculates team points earned based on Result.

        Returns:
            int: Number of team points earned.

        """
        if self.value == 1:
            val = 3
        elif self.value == 2:
            val = 4
        elif self.value == 3:
            val = 5
        elif self.value == 4:
            val = 6
        else:  # loss
            val = 0
        return val


@attr.s(auto_attribs=True, eq=False, order=False, slots=True)
class Mark(object):
    """Mark object which acts as a Meta class for str and int inputs.

    This class should be used whenever validation on a class field is
    stonger than simple type validation. Prompts will be provided for
    established classes.

    Args:
        tag String or Integer value for the Mark.
        isvalid: Whether tag is valid or invalid, default to True.
        msg: Message for tag, defaults to empty string, should be changed if isvalid
        is False.

    """

    tag: Union[str, int] = attr.ib()
    isvalid: bool = attr.ib(default=True, init=False, validator=instance_of(bool))
    msg: str = attr.ib(default="", init=False, repr=False, validator=instance_of(str))

    @tag.validator
    def check_tag(self, attribute, value):
        """Attrs based validator function for tag attribute.

        Raises:
            TypeError: TypeError if tag value is not int or str type.

        """
        if not isinstance(value, str) and not isinstance(value, int):
            raise TypeError(
                f"`tag` value must be of type 'int' or 'str', got {type()!r}."
            )


@attr.s(auto_attribs=True, order=False, eq=False, slots=True)
class CollegeLabel(Mark):
    """Label class for College scoring event labels.

    Args:
        point_value: Numeric point value for label, different than tag value.

    """

    point_value: int = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        """Post init hook function.

        This function checks if the label tag is considered a valid label based
        on college (folkstyle) ruleset.  If not it adjusts the 'isvalid' and 'msg'
        attributes accordingly.

        """
        if self.tag in self.valid_labels:
            self.point_value = self.points_dict[self.tag]
        else:  # invalid tag
            message = (
                f"Invalid tag for 'College Label'. Expected one of "
                f"{*self.valid_labels,}, got {self.tag}."
            )
            self.point_value = 0
            self.isvalid = False
            self.msg = message

    @property
    def valid_labels(self) -> Set:
        """Set of valid scoring event labels based on current college ruleset.

        Returns:
            Set: Scoring events.
        """
        return {
            "START",
            "T2",
            "E1",
            "R2",
            "N2",
            "N4",
            "C",
            "P1",
            "P2",
            "WS",
            "S1",
            "S2",
            "RO1",
            "BOT",
            "TOP",
            "NEU",
            "DEFER",
            "Fall",
            "Tech Fall",
            "Forfeit",
            "Default",
        }

    @property
    def points_dict(self) -> Dict:
        """Dictionary of valid scoring event labels and their point values.

        Returns:
            Dict: Dictionary of labels and their corresponding point values.
        """
        return {
            "START": 0,
            "T2": 2,
            "E1": 1,
            "R2": 2,
            "N2": 2,
            "N4": 4,
            "C": 0,
            "P1": 1,
            "P2": 2,
            "WS": 0,
            "S1": 1,
            "S2": 2,
            "RO1": 1,
            "BOT": 0,
            "TOP": 0,
            "NEU": 0,
            "DEFER": 0,
            "Fall": 0,
            "Tech Fall": 0,
            "Forfeit": 0,
            "Default": 0,
        }


@attr.s(auto_attribs=True, order=False, eq=False, slots=True)
class HSLabel(Mark):
    """Label class for High School scoring event labels.

    Args:
        point_value: Numeric point value for label, different than tag value.

    """

    point_value: int = attr.ib(validator=instance_of(int), init=False, repr=False)

    def __attrs_post_init__(self):
        """Post init hook function.

        This function checks if the label tag is considered a valid label based
        on high school (folkstyle) ruleset.  If not it adjusts the 'isvalid' and 'msg'
        attributes accordingly.

        """
        if self.tag in self.valid_labels:
            self.point_value = self.points_dict[self.tag]
        else:  # invalid tag
            message = (
                f"Invalid tag for 'High School Label'. Expected one of "
                f"{*self.valid_labels,}, got {self.tag}."
            )
            self.point_value = 0
            self.isvalid = False
            self.msg = message

    @property
    def valid_labels(self) -> Set:
        """Set of valid scoring event labels based on current college ruleset.

        Returns:
            Set: Scoring events.
        """
        return {
            "START",
            "T2",
            "E1",
            "R2",
            "N2",
            "N3",
            "C",
            "P1",
            "P2",
            "WS",
            "S1",
            "S2",
            "BOT",
            "TOP",
            "NEU",
            "DEFER",
            "Fall",
            "Tech Fall",
            "Forfeit",
            "Default",
        }

    @property
    def points_dict(self) -> Dict:
        """Dictionary of valid scoring event labels and their point values.

        Returns:
            Dict: Dictionary of labels and their corresponding point values.
        """
        return {
            "START": 0,
            "T2": 2,
            "E1": 1,
            "R2": 2,
            "N2": 2,
            "N3": 3,
            "C": 0,
            "P1": 1,
            "P2": 2,
            "WS": 0,
            "S1": 1,
            "S2": 2,
            "BOT": 0,
            "TOP": 0,
            "NEU": 0,
            "DEFER": 0,
            "Fall": 0,
            "Tech Fall": 0,
            "Forfeit": 0,
            "Default": 0,
        }



from wrestling.wrestlers import convert_to_title, Wrestler
from wrestling.base import Mark, YEARS


def test_convert_to_title():
    names = ['Nsnfk Afansjfsa', ' fs fasfsa ', 'NFASKJFAFA']
    for name in names:
        assert name.title().strip() == convert_to_title(name)


wrestler = Wrestler(name='Nick Anthony', team='Eagles', grade=Mark('Fr.'))

class TestWrestler:
    def test_grade(self):
        assert wrestler.grade in YEARS





#! /usr/bin/python

"""Module for Scoring Events.

This module builds the Scoring Events base class with validation
for its fields. When validation is stronger than simple type
validation, the Mark class is used in replace of traditional
str or int classes to track accuracy.

Example:
    >>>scoring_event = CollegeScoring(
        time_stamp=time(hour=0, minute=5, second=15),
        initiator='red',
        focus_color='green',
        period=2,
        label=CollegeLabel('T2')
    )

Todo:
    * Find a way to validate period based on timestamp.

"""

import abc
from datetime import time
from typing import Dict, Union

import attr
from attr.validators import in_, instance_of

from wrestling import base
from wrestling.base import CollegeLabel, HSLabel


@attr.s(slots=True, eq=True, order=True, auto_attribs=True, kw_only=True)
class ScoringEvent(object):
    """Scoring Event base class for any scoring action in a match.

    Args:
        time_stamp (time): Time the action occured.
        initiator (str): Who initiated the action, red or green.
        focus_color (str): Focus of the match.
        period (int): Period in which the action occured.

    Raises:
        ValueError: Hour parameter of time_stamp cannot be non-zero.
        ValueError: 'focus_color' and 'initiator' must be either 'red' or 'green'

    """

    time_stamp: Union[time, str] = attr.ib(validator=instance_of((time, str)), order=True)
    initiator: str = attr.ib(
        validator=[instance_of(str), in_(("red", "green"))], order=False,
    )
    focus_color: str = attr.ib(
        validator=[instance_of(str), in_(("red", "green"))], order=False, repr=False
    )
    period: int = attr.ib(validator=instance_of(int), order=False, repr=False)
    focus_score: int = attr.ib(default=0, init=False, order=False, eq=False)
    opp_score: int = attr.ib(default=0, init=False, order=False, eq=False)

    @property
    @abc.abstractmethod
    def label(self):
        """Label of the action that occurred."""
        pass

    @time_stamp.validator
    def check_time_stamp(self, attribute, val):
        """Attrs validator, checks timestamp that hour is not zero."""
        if str(val).split(':')[0] != '00':
                raise ValueError(f"`hour` field of timestamp must be 0 (zero).")

    @property
    def formatted_time(self) -> str:
        """String formatted time.

        Returns:
            str: Minute:Second string formatted time_stamp.

        """
        return str(self.time_stamp)[3:]

    @property
    def formatted_label(self) -> str:
        """String formatted label.

        Raises:
            ValueError: initiator and focus_color must be either red or green.

        Returns:
            str: Label with focus (f) or opponent (o) prefix.

        """
        if self.focus_color == self.initiator:
            return f"f{self.label.tag}"
        elif self.focus_color != self.initiator:
            return f"o{self.label.tag}"
        else:
            raise ValueError(
                f'Expected "red" or "green" '
                f"for `focus_color` AND "
                f"`initiator`, got {self.focus_color} and "
                f"{self.initiator}."
            )

    def to_dict(self) -> Dict[str, Union[int, str, CollegeLabel, HSLabel]]:
        """Converts instance to dict.

        Returns:
            Dict: Dictionary representation of Scoring Event instance.
        """
        return dict(
            time=self.formatted_time,
            period=self.period,
            str_label=self.formatted_label,
            label=self.label,
            focus_score=self.focus_score,
            opp_score=self.opp_score,
        )


@attr.s(slots=True, eq=True, order=True, auto_attribs=True, kw_only=True)
class CollegeScoring(ScoringEvent):
    """College version of a Scoring Event.

    Args:
        label (CollegeLabel): Label for the action.

    """

    label: base.CollegeLabel = attr.ib(
        validator=instance_of(base.CollegeLabel), order=False, repr=lambda x: x.tag
    )


@attr.s(slots=True, eq=True, order=True, auto_attribs=True, kw_only=True)
class HSScoring(ScoringEvent):
    """High School version of a Scoring Event.

    Args:
        label (HSLabel): Label for the action.
    """

    label: base.HSLabel = attr.ib(
        validator=instance_of(base.HSLabel), order=False, repr=lambda x: x.tag
    )
